from django.contrib.auth.models import User
from django.db import models, IntegrityError
from django.utils.text import slugify
import os


def get_upload_path(instance, filename):
    return 'entries/%s%s' % (instance.slug, os.path.splitext(filename)[1])


class EntryQuerySet(models.QuerySet):
    def by_author(self, author):
        return self.filter(author=author)

    def by_category(self, category_slug):
        return self.filter(category__slug=category_slug)


class Entry(models.Model):
    title = models.CharField(max_length=256)
    slug = models.SlugField(unique=True)
    create_time = models.DateTimeField(auto_now_add=True)
    content = models.TextField()
    author = models.ForeignKey(User)
    image = models.ImageField(upload_to=get_upload_path, null=True, blank=True)
    category = models.ForeignKey("entries.Category")

    objects = EntryQuerySet.as_manager()

    class Meta:
        verbose_name_plural = "Entries"
        ordering = ('-create_time', )

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        # create slug only if this is a new entry.
        if not self.id:
            self.slug = slugify(self.title)

        # try to save, if there's an integrity error with the slug then
        # add a increasing number, until saved correctly.
        tries = 0
        while True:
            try:
                return super(Entry, self).save(*args, **kwargs)
            except IntegrityError:
                tries += 1
                self.slug = "%s_%d" % (self.slug, tries)


class Category(models.Model):
    name = models.CharField(max_length=64)
    name_plural = models.CharField(max_length=66)
    slug = models.CharField(max_length=64, unique=True)

    class Meta:
        verbose_name_plural = "Categories"

    def __unicode__(self):
        return self.name + " (" + self.name_plural + ")"
